@@ -4,7 +4,7 @@ use strict;
 use warnings;
 use 5.008001;
 
-our $VERSION = '0.56';
+our $VERSION = '0.57';
 
 use Carp qw(croak);
 use URI ();
@@ -47,7 +47,7 @@ sub playback_url {
 
     my $data = $self->prepare_download($video_id);
     my $fmt  = $args->{fmt} || $data->{fmt} || DEFAULT_FMT;
-    my $video_url = $data->{video_url_map}{$fmt}{url} || croak "this video has not supported fmt: $fmt";
+    my $video_url = $data->{video_url_map}{$fmt}{url} || croak "this video does not offer format (fmt) $fmt";
 
     return $video_url;
 }
@@ -92,7 +92,7 @@ sub _format_filename {
 sub _is_supported_fmt {
     my ($self, $video_id, $fmt) = @_;
     my $data = $self->prepare_download($video_id);
-    $data->{video_url_map}{$fmt}{url} ? 1 : 0;
+    defined($data->{video_url_map}{$fmt}{url}) ? 1 : 0;
 }
 
 sub _default_cb {
@@ -168,8 +168,11 @@ sub _fetch_title {
 sub _fetch_user {
     my ($self, $content) = @_;
 
-    my ($user) = $content =~ /<span class="yt-user-name\s+?" dir="ltr">([^<]+)<\/span>/ or return;
-    return decode_entities($user);
+	if( $content =~ /<span class="yt-user-name [^>]+>([^<]+)<\/span>/ ){
+		return decode_entities($1);
+	}else{
+		return;
+	}	
 }
 
 sub _fetch_video_url_map {
@@ -223,7 +226,7 @@ sub _get_args {
             croak 'Video not available in your country';
         }
         elsif ($line =~ /^.+ytplayer\.config\s*=\s*({.*})/) {
-            $data = JSON->new->utf8(1)->decode($1);
+            ($data, undef) = JSON->new->utf8(1)->decode_prefix($1);
             last;
         }
     }
@@ -248,37 +251,36 @@ sub _sigdecode {
     my @s = @_;
 
     # based on youtube_dl/extractor/youtube.py from yt-dl.org
-    if (@s == 92) {
-	return ($s[25], @s[3..24], $s[0], @s[26..41], $s[79], @s[43..78], $s[91], @s[80..82]);
+    if (@s == 93) {
+        return (reverse(@s[30..86]), $s[88], reverse(@s[6..28]));
+    } elsif (@s == 92) {
+        return ($s[25], @s[3..24], $s[0], @s[26..41], $s[79], @s[43..78], $s[91], @s[80..82]);
+    } elsif (@s == 91) {
+        return (reverse(@s[28..84]), $s[86], reverse(@s[6..26]));
     } elsif (@s == 90) {
-	return ($s[25], @s[3..24], $s[2], @s[26..39], $s[77], @s[41..76], $s[89], @s[78..80]);
+        return ($s[25], @s[3..24], $s[2], @s[26..39], $s[77], @s[41..76], $s[89], @s[78..80]);
+    } elsif (@s == 89) {
+        return (reverse(@s[79..84]), $s[87], reverse(@s[61..77]), $s[0], reverse(@s[4..59]));
     } elsif (@s == 88) {
-        return ($s[48], reverse(@s[68..81]), $s[82], reverse(@s[63..66]), $s[85],
-                reverse(@s[49..61]), $s[67], reverse(@s[13..47]), $s[3],
-                reverse(@s[4..11]), $s[2], $s[12]);
+        return (@s[7..27], $s[87], @s[29..44], $s[55], @s[46..54], $s[2], @s[56..86], $s[28]);
     } elsif (@s == 87) {
-        return (@s[4..22], $s[86], @s[24..84]);
+        return (@s[6..26], $s[4], @s[28..38], $s[27], @s[40..58], $s[2], @s[60..86]);
     } elsif (@s == 86) {
-        return (@s[2..62], $s[82], @s[64..81], $s[63]);
+        return (@s[4..30], $s[3], @s[32..84]);
     } elsif (@s == 85) {
-       return (@s[2..7], $s[0], @s[9..20], $s[65], @s[22..64], $s[84], @s[66..81], $s[21]);
+        return (@s[3..10], $s[0], @s[12..54], $s[84], @s[56..83]);
     } elsif (@s == 84) {
-        return (reverse(@s[37..83]), $s[2], reverse(@s[27..35]), $s[3],
-                reverse(@s[4..25]), $s[26]);
+        return (reverse(@s[71..78]), $s[14], reverse(@s[38..69]), $s[70], reverse(@s[15..36]), $s[80], reverse(@s[0..13]));
     } elsif (@s == 83) {
-        return ($s[6], @s[3..5], $s[33], @s[7..23], $s[0], @s[25..32], $s[53], @s[34..52], $s[24], @s[54..82]);
+        return (reverse(@s[64..80]), $s[0], reverse(@s[1..62]), $s[63]);
     } elsif (@s == 82) {
-        return ($s[36], reverse(@s[68..79]), $s[81], reverse(@s[41..66]), $s[33],
-                reverse(@s[37..39]), $s[40], $s[35], $s[0], $s[67],
-                reverse(@s[1..32]), $s[34]);
+        return (reverse(@s[38..80]), $s[7], reverse(@s[8..36]), $s[0], reverse(@s[1..6]), $s[37]);
     } elsif (@s == 81) {
-        return ($s[56], reverse(@s[57..79]), $s[41], reverse(@s[42..55]), $s[80],
-		reverse(@s[35..40]), $s[0], reverse(@s[30..33]), $s[34],
-		reverse(@s[10..28]), $s[29], reverse(@s[1..8]), $s[9]);
+        return ($s[56], reverse(@s[57..79]), $s[41], reverse(@s[42..55]), $s[80], reverse(@s[35..40]), $s[0], reverse(@s[30..33]), $s[34], reverse(@s[10..28]), $s[29], reverse(@s[1..8]), $s[9]);
+    } elsif (@s == 80) {
+        return (@s[1..18], $s[0], @s[20..67], $s[19], @s[69..79]);
     } elsif (@s == 79) {
-        return ($s[54], reverse(@s[55..77]), $s[39], reverse(@s[40..53]), $s[78],
-		reverse(@s[35..38]), $s[0], reverse(@s[30..33]), $s[34],
-		reverse(@s[10..28]), $s[29], reverse(@s[1..8]), $s[9]);
+        return ($s[54], reverse(@s[55..77]), $s[39], reverse(@s[40..53]), $s[78], reverse(@s[35..38]), $s[0], reverse(@s[30..33]), $s[34], reverse(@s[10..28]), $s[29], reverse(@s[1..8]), $s[9]);
     }
 
     return ();    # fail
@@ -299,9 +301,9 @@ sub _parse_stream_map {
         my $uri = URI->new;
         $uri->query($stuff);
         my $query = +{ $uri->query_form };
-        my $sig = $query->{sig} || _getsig($query->{s});
+        my $sig = $query->{sig} || ($query->{s} ? _getsig($query->{s}) : undef);
         my $url = $query->{url};
-        $fmt_url_map->{$query->{itag}} = $url.'&signature='.$sig;
+        $fmt_url_map->{$query->{itag}} = $url . ($sig ? '&signature='.$sig : '');
     }
 
     return $fmt_url_map;
@@ -316,7 +318,7 @@ sub ua {
 
 sub _suffix {
     my $fmt = shift;
-    return $fmt =~ /43|44|45/    ? 'webm'
+    return $fmt =~ /43|44|45|46/ ? 'webm'
          : $fmt =~ /18|22|37|38/ ? 'mp4'
          : $fmt =~ /13|17/       ? '3gp'
          :                         'flv'
@@ -385,7 +387,8 @@ WWW::YouTube::Download - Very simple YouTube video download interface
 
 =head1 DESCRIPTION
 
-WWW::YouTube::Download is a download video from YouTube.
+WWW::YouTube::Download is a library to download videos from YouTube. It relies entirely on
+scraping a video's webpage and does not use YT's /get_video_info URL space.
 
 =head1 METHODS
 
@@ -575,7 +578,9 @@ Plese use github issues: L<< https://github.com/xaicron/p5-www-youtube-download/
 
 =head1 SEE ALSO
 
+L<WWW::YouTube::Info> and L<WWW::YouTube::Info::Simple>.
 L<WWW::NicoVideo::Download>
+L<http://rg3.github.io/youtube-dl/>
 
 =head1 LICENSE
 
